"""notebooksalamode.Datasets"""

import hashlib

class Datasets(object):
    """Container providing mulitple lookup methods for each registered Dataset.

    ... by Dataset name
    ... by Dataset token
    ... by Dataset insert order to provide backwards compatibility with early
        versions of our notebooks.
    """
    def __init__(self):
        self.by_index = []
        self.by_token = {}
        self.by_name = {}

    def __getitem__(self, key):
        if isinstance(key, basestring):
            if key in self.by_token:
                return self.by_token[key].data

            hkey = hashlib.sha256(key).hexdigest()
            if hkey in self.by_name:
                return self.by_name[hkey].data
            raise KeyError
        else:
            if key >= 0 and key <= len(self.by_index):
                return self.by_index[key].data
            raise IndexError

    def add_dataset(self, dset):
        """Add the provided dataset"""
        self.by_index.append(dset)
        self.by_token[dset.token] = dset
        self.by_name[hashlib.sha256(dset.name).hexdigest()] = dset

    def _repr_html_(self):
        output = (
            "<table class='dataframe'>\n"
            "<thead>\n"
            "<tr style='text-align: right'>"
            "<th>index</th>"
            "<th>qref</th>"
            "<th>name</th>"
            "<th>rows</th>"
            "<th>columns</th>"
            "</tr></thead>\n"
            "<tbody>\n")

        for i, dset in enumerate(self.by_index):
            rows, cols = dset.data.shape
            style = "text-color: #ccc" if dset.error else ""
            if dset.error:
                output += "<tr style='{}'><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td></tr>\n".format(
                    style, i, dset.token, dset.name, rows, cols)
            else:
                output += "<tr><td>{}</td><td>{}</td><td>{}</td><td>{}</td><td>{}</td></tr>\n".format(
                    i, dset.token, dset.name, rows, cols)

        output += "</tbody>\n</table>"
        return output
